/* 
  * Glenn Fowler 
  * AT&T Research 
  * 9x9 sudoku backtrack solver -- no dancing 
  * option -u to verify only one solution 
  * puzzles on the standard input 
  * coded for simplicity over speed 
  */ 
 
 #include <stdio.h> 
 
 typedef struct Grid_s { 
         unsigned short  rcb[27], cell[81]; 
 } Grid_t; 
 
 typedef struct Try_s { 
         int     candidates, cell, free, guess, index; 
 } Try_t; 
 
 static const char       name[] = ".123456789"; 
 
 static unsigned char    constraint[81][3]; 
 static unsigned char    ident[1<<9]; 
 static unsigned char    count[1<<9]; 
 static unsigned char    option[1<<8]; 
 
 static unsigned short   next[1<<9]; 
 static unsigned short   token[1<<8]; 
 
 static Grid_t           base; 
 
#define MASK(g,i) (g.rcb[constraint[i][0]]&g.rcb[constraint[i][1]]&g.rcb[constraint[i][2]]) 
#define PROP(g,i,x) (g.rcb[constraint[i][0]]^=(x),g.rcb[constraint[i][1]]^=(x),g.rcb[constraint[i][2]]^=(x)) 
#define MOVE(g,i,x)     (g.cell[i]=(x),PROP(g,i,x)) 
#define UNDO(g,i,x)     (g.cell[i]=(0),PROP(g,i,x)) 
 
 int 
 backtrace (char *puzzle_in,char *puzzle_out) { 
         register int    i, j, k, m, n, x; 
         int             level, depth, solutions; 
         //char*           s; 
         Grid_t          grid; 
         Try_t           try[81]; 
 /*
         while ((s = *++argv)) 
                 while ((i = *s++)) 
                         option[i] = 1; 
*/
         for (i = 1; i <= 9; i++) 
                 ident[token[(int)name[i]] = (1<<(i-1))] = i; 
         token['.'] = token['0'] = token['_'] = 0x1ff; 
         for (i = 0; i < 27; i++) 
                 base.rcb[i] = 0x1ff; 
         for (i = 0; i < 1<<9; i++) { 
                 k = 0; 
                 for (j = 1; j < 1<<9; j <<= 1) 
                         if ((i & j) && !k++) 
                                 next[i] = j; 
                 count[i] = k; 
         } 
         for (k = i = 0; i < 9; i++) { 
                 n = 18 + ((i / 3) * 3); 
                 for (j = 9; j < 18; j++) { 
                         constraint[k][0] = i; 
                         constraint[k][1] = j; 
                         constraint[k][2] = n + ((j / 3) % 3); 
                         k++; 
                 } 
         } 
         for (;;) { 
                 grid = base; 
                 i = depth = 0; 
                 while ((x = (int) *puzzle_in++)) 
                         if ((j = token[x])) { 
                                 if (j == 0x1ff) 
                                         try[depth++].free = i; 
                                 else 
                                         MOVE(grid, i, j); 
                                 if (++i >= 81) 
                                         break; 
                         } 
                         else if ((x == '"') || ((x == '#' || x == '/') && (x = '\n')) || (x == '<' && (x = '>'))) 
                                 while ((j = (int) *puzzle_in++) && j != x); 
                 if (i < 81) 
                 { 
                         if (x == 0) 
                                 break; 
                         continue; 
                 } 
                 depth--; 
                 level = solutions = 0; 
                 for (;;) { 
                         m = 10; 
                         for (k = level; k <= depth; k++) { 
                                 i = try[k].free; 
                                 if (!(x = grid.cell[i] = MASK(grid, i))) { 
                                         m = 11; 
                                         break; 
                                 } 
                                 if (m > (n = count[x])) { 
                                         try[level+1].index = k; 
                                         try[level+1].candidates = x; 
                                         if ((m = n) == 1) 
                                                 break; 
                                 } 
                         } 
                         if (m < 10) { 
                                 k = try[level+1].index; 
                                 try[level+1].cell = try[k].free; 
                                 if (k != level) { 
                                         j = try[level].free; 
                                         try[level].free = try[k].free; 
                                         try[k].free = j; 
                                 } 
                                 level++; 
                         } 
                         else { 
                                 if (m == 10) { 
                                         if (solutions++) { 
                                                 printf(" multiple solutions\n"); 
                                                 break; 
                                         } 
                                         for (i = 0; i < 81; i++) 
	                                           *puzzle_out++=name[ident[grid.cell[i] 
 ]]; 
                                         if (!option['u']) 
                                                 break; 
                                 } 
                                 UNDO(grid, try[level].cell, try[level].guess); 
                         } 
                         while (!(x = next[try[level].candidates])) { 
                                 if (!--level) { 
                                         if (!solutions) 
                                                 printf("no solution\n"); 
                                         return (0);//goto done; 
                                 } 
                                 UNDO(grid, try[level].cell, try[level].guess); 
                         } 
                         try[level].candidates ^= x; 
                         try[level].guess = x; 
                         MOVE(grid, try[level].cell, x); 
                 } 
//         done: 
//                 putchar('\n'); 
         } 
         return 0; 
 } 
