/*

 Gestione BitWise Logic
 questo sorgente e' parte integrante 
 del Progetto xSudoky.
 Scritto da Max Cavallo 20/Luglio/2007
 ixamit@gmail.com

 Struttura Bit per cella
 -----------------------
  0.. 8 	rappresentano i candidati (o numero) presenti
  9		non utilizzato
 10		E' candidato ? (0==numero 1==candidato)
 11		E' fisso ?

*/

#include "sudoky.h"
 
char *DecToBin(char *pSave,int   n)
{
	//
	// Conversione Decimale -> Binaria
	//
	#define   INT   16
	int i;

	for(i = 0;i<INT;i++)
	{
		if(0==i && (n & 0x8000) != 0)
		{
        		pSave[i]   =   '1';
			n=-n;
			n <<= 1;
			continue;
		}
                pSave[i] = ((n   &0x8000) != 0) +'0';
                n   <<=   1;
	}
	pSave[i]=0;
	return pSave;
}

int *CoordinateBlocco2yx (int Blocco, int Posizione, int *y, int *x)
{
	//
	// Trasforma le coordinate blocco in y,x
	// restituisce anche il ptr alla cella
	//
	*y=((Blocco/DIVY)*DIVY) + (Posizione/DIVY);
	*x=((Blocco%DIVX)*DIVX) + (Posizione%DIVX);

	return (&SUD_BUF[*y][*x]);
}

int *Coordinateyx2Blocco (int y,int x, int *Blocco, int *Posizione)
{
	//
	// Trasforma le coordinate y,x in Blocco
	// restituisce anche il ptr alla cella
	//
	*Blocco   =((y/DIVY)*DIVY) + (x/DIVX);
	*Posizione=((y % DIVY)*DIVY) + (x % DIVX);

	return (&SUD_BUF[y][x]);
}

int GetCandidato(int bits)
{
	int candi;

	if (!bits) return 0;

	for (candi=0;candi<SUD_TAB;candi++)
		if (bits & (1<<candi)) break;

	return (candi+1);
}

char *GetCandidati(int y, int x)
{
	//
	// Restituisce i candidati 
	// sulle coordinate y,x
	// 
	static char candidati[SUD_TAB+1];	// '\0' need 4 Display
	int j,z;
	int bits;

	bits=SUD_BUF[y][x];

	for (z=0,j=0;j<SUD_TAB;j++)
		if (bits & (1<<j))
			candidati[z++]=(j+1)+'0';
	candidati[z]='\0';

	return (candidati);
}

char *GetCandidati2(int b)
{
	//
	// Restituisce i candidati 
	// sulle coordinate y,x
	// 
	static char candidati[SUD_TAB+1];	// '\0' need 4 Display
	int j,z;
	int bits;

	bits=b;

	for (z=0,j=0;j<SUD_TAB;j++)
		if (bits & (1<<j))
			candidati[z++]=(j+1)+'0';
	candidati[z]='\0';

	return (candidati);
}

int IsFixed (int y,int x)
{
	return (SUD_BUF[y][x] & FIXED_MSK);
}

int IsCandi (int y,int x)
{
	return (SUD_BUF[y][x] & CANDI_MSK);
}

int IsEvidence (int y, int x)
{
	return (SUD_BUF[y][x] & (1<<(EVIDENCE-1)));
}

int IsCandiExist (int y, int x, int candidato)
{
	//
	// Restituisce VERO
	// se candidato esiste 
	// in posizione y,x
	//
	return (SUD_BUF[y][x] & (1<<(candidato-1)));
}

int ContaBits (int b)
{
	//
	// Conta il numero di bits accesi
	//
	int n;
	int iscandi=b & CANDI_MSK;
	int isfixed=b & FIXED_MSK;

	for (n=0;b;++n) b&=b-1;

	if (iscandi && n) n--;	
	if (isfixed && n) n--;
	return n;
}

int IsOnly1Candi (int b)
{
	return (ContaBits (b)==1);
}

int CandiExist (int cell,int candi)
{
	return (cell & (1 << (candi-1)));
}

int *AddCandidato (int *cella, int candidato)
{
	//
	// Aggiunge il candidato al ptr cella
	//
	*cella |= (1 << (candidato-1));
	return (cella);
}

int *RemoveCandidato (int *cella, int candidato)
{
	//
	// Rimuve il candidato dal ptr cella
	//
	if (*cella & CANDI_MSK)
		*cella &= ~(1 << (candidato-1));

	return (cella);
}

int RemoveCandidatoRiga (int riga, int candidato)
{
	//
	// Cancella il candidato dalla Riga
	//
	int x;

	for (x=0;x<SUD_TAB;x++)
		RemoveCandidato (&SUD_BUF[riga][x], candidato);

	return (0);
}

int RemoveCandidatoColonna (int colonna, int candidato)
{
	//
	// Cancella il candidato dalla Colonna
	//
	int y;

	for (y=0;y<SUD_TAB;y++)
		RemoveCandidato (&SUD_BUF[y][colonna], candidato);

	return (0);
}

int RemoveCandidatoBlocco (int blocco, int candidato)
{
	//
	// Cancella il candidato dal Blocco
	//
	int p,x,y;
	int *cella;

	for (p=0;p<SUD_TAB;p++)
	{
		cella=CoordinateBlocco2yx (blocco, p, &y,&x);
		RemoveCandidato (cella, candidato);
	}

	return (0);
}

int RemoveCandidati ()
{
	//
	// Cancella candidati su righe colonne e blocchi
	//
	int riga,colonna,blocco,posizione,numero;
	int *cella;
	char *p;
	for (riga=0;riga<SUD_TAB;riga++)
	{
		for (colonna=0;colonna<SUD_TAB;colonna++)
		{
			cella=&SUD_BUF[riga][colonna];
			if (!(*cella & CANDI_MSK))
			{
				p=GetCandidati(riga,colonna);	// Prende Numero
				numero=p[0]-'0';

				RemoveCandidatoRiga (riga,numero);
				RemoveCandidatoColonna (colonna,numero);
				Coordinateyx2Blocco (riga,colonna,&blocco,&posizione);
				RemoveCandidatoBlocco (blocco, numero);		
			}
		}
	}
	return (0);		
}

int *SetFlagCandidato (int *cella, int Mask)
{
	//
	// Setta Flags Mask al ptr cella
	//
	*cella |= Mask;
	return (cella);
}

int *UnsetFlagCandidato (int *cella, int Mask)
{
	//
	// Azzera Flags Mask al ptr cella
	//
	*cella &= ~ Mask;
	return (cella);
}

int IsMouseOver (int y,int x)
{
	int mo_x=0;
	int mo_y=0;
	if (MOUSEOVER[0]=='X')
	{
		mo_y=MOUSEOVER[1]-'0';
		mo_x=MOUSEOVER[2]-'0';
	}
	return ((mo_x==x && mo_y==y) ? 1 : 0);
}

